/**
  VGA Graphic Demo with Parallax VGA_512x384_Bitmap Interface
  for DemoBoard Wiring
  Reinhard / January 2012

  update May 2019
  The screen is refreshed cyclically.
  A dotted zero line is output.
  Every 2 seconds you switch between DC coupling and AC coupling.
  The gain at AC is greater.	
  The AC coupling is realized with an IIR high pass.
*/

#include <stdio.h>
#include <string.h>
#include <propeller.h> 
#include "fixed.h"

#define N 512
#define q 16                  
////////////////////////////////////////////////////////////////////////
//  Mailbox
////////////////////////////////////////////////////////////////////////
static unsigned int sync; 
static unsigned int reg_dira;
static unsigned int reg_vcfg;
static unsigned int * pixel_ptr;
static unsigned int * color_ptr;

static int y_sample;
static int samples[512];

int n;               // common used index  
int T1,T2,T3,T4;     // Help Terms 	
int A[2];            // Forward Coefficients
int B[2];            // Reverse Coefficients
int xx[2];            // Forward stage
int yy[2];            // Reverse stage  
////////////////////////////////////////////////////////////////////////
unsigned int   PIXEL_BUFFER[6144];
unsigned int   COLOR_BUFFER[192];
////////////////////////////////////////////////////////////////////////
void msleep(int t)
{
    waitcnt((CLKFREQ/1000)*t+CNT);
}

////////////////////////////////////////////////////////////////////////
void plot( unsigned int x, unsigned int y)
{

	int tmp = x % 32;

	if(x >= 0 && x < 512 && y >= 0 && y < 384)
	{
	  PIXEL_BUFFER[(x/32) + (y*16)] |=  1<<tmp; 
	}
}
////////////////////////////////////////////////////////////////////////
void clear( unsigned int x, unsigned int y)
{

	int tmp = x % 32;

	if(x >= 0 && x < 512 && y >= 0 && y < 384)
	{
	  PIXEL_BUFFER[(x/32) + (y*16)] =0; 
	}
}
////////////////////////////////////////////////////////////////////////
int start(unsigned int *ptr)  
{
    extern unsigned int binary_vga_512x384_bitmap_dat_start[];
    return cognew(&binary_vga_512x384_bitmap_dat_start, ptr);
}
////////////////////////////////////////////////////////////////////////
int start2(unsigned int *ptr)  
{
    extern unsigned int binary_microphone_to_headphones_dat_start[];
    return cognew(&binary_microphone_to_headphones_dat_start, ptr);
}
////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
void main (void)
{
  unsigned int x,i,c;

	// Preset the mailbox parameters
  sync = 0;
	reg_dira = 0x00FF0000;
	reg_vcfg = 0x200000FF + (16 << 6);  

  // Fill Color Buffer
	for(i = 0; i < 192; i++)
	{
		COLOR_BUFFER[i] = 0x30003000 ;
	}
	
  pixel_ptr = PIXEL_BUFFER;
	color_ptr = COLOR_BUFFER;

	// start the vga Driver
	start(&sync);
	// start the mic driver
	start2(&y_sample);

  A[0] = TOFIX(0.9695,q);    	// Setup Filter
  A[1] = TOFIX(-0.9695,q);		// Coefficients
  B[0] = TOFIX(1.0,q);
  B[1] = TOFIX(-0.9391,q);

  c=0;
/*
  Graphic Demo :
*/
	while(1)
	{
    c++;    
    xx[0] = 0;
    xx[1] = 0;
  
    yy[0] = 0;
    yy[1] = 0;  
    for(i=0;i<512;i++)
    { 
      samples[i] = y_sample; 
      usleep(10);
    }	

//	IIR Highpass 1. Order
//	y[0]   = A[0] * x[0] + A[1] * x[1]  - B[1] * y[1];
//  out    = B[0] * y[0];
////////////////// Filter Loop //////////////////////
  for(n=0;n<N;n++)     
  {
    if(c<10)
    {    
    xx[0]= 	samples[n];
    T1 = 	FMUL(B[1],yy[1],q);
    T2 = 	FMUL(A[1],xx[1],q);
    T3 = 	FMUL(A[0],xx[0],q);
    T4 = 	FADD(T3,T2);
    yy[0] = 	FSUB(T4,T1);
    samples[n] = 	FMUL(B[0],yy[0],q);
    xx[1]= 	xx[0];
    yy[1]= 	yy[0]; 
    samples[n]= 192 - (samples[n]>>10);
    }
    else
      samples[n]= 192 - (samples[n]>>24);
  }
////////////////// filter end /////////////////////
    if(c==20) c=0;
    // plot samples
 	  for(x = 0;x < 512;x++)
 	  {
      plot(x,samples[x] ); 
      if(x%10 == 0)plot(x,192);//dotted ground line
    }
    
    msleep(200);

    //clear screen
    for(x = 0;x < 512;x++)
    {
      clear(x,samples[x]);
    }
  }
}
////////////////////////////////////////////////////////////////////////

